#ifdef _VERTEX_

void main()
{
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}

#else

vec4  ColorMapTerra(vec3 point, float height, float slope)
{
    vec3  p = point * mainFreq + Randomize;
    vec4  col;
    noiseOctaves = 6;
    vec3  distort = p * 2.3 + 3.5 * Fbm3D(p * 0.6);
    vec2  cell = Cell3Noise2Color(distort, col);
    float biome = col.r;
    /*float biomeScale = saturate(2.0 * (pow(abs(cell.y - cell.x), 0.7) - 0.05));
    vec4  colorOverlay;
    if (biome < dunesFraction)
        colorOverlay = vec4(1.0, 1.0, 0.0, 0.0);
    else if (biome < hillsFraction)
        colorOverlay = vec4(0.0, 1.0, 0.0, 0.0);
    else if (biome < hills2Fraction)
        colorOverlay = vec4(0.0, 1.0, 0.5, 0.0);
    else if (biome < canyonsFraction)
        colorOverlay = vec4(1.0, 0.0, 0.0, 0.0);
    else
        colorOverlay = vec4(1.0, 1.0, 1.0, 0.0);*/

    noiseOctaves = 2;
    distort  = 0.035 * Fbm3D(p * 53.1);
    distort += 0.350 * Fbm3D(p *  5.8);
    cell = Cell3Noise2(3.1 * p + distort);
    float pseudoRivers = saturate(1.0 - abs(cell.y - cell.x) * 60.0);
    pseudoRivers *= 1.0 - saturate((height / seaLevel - 1.0) * 2.0);

    Surface surf;

    // Assign a climate type
    noiseOctaves = (cloudsStyle == 1.0) ? 5.0 : 12.0;
    noiseH          = 0.7;
    noiseLacunarity = 2.218281828459;
    noiseOffset     = 0.8;
    float climate, latitude, dist;
    if (tidalLock == 0.0)
    {
        latitude = abs(point.y);
        latitude += 0.15 * (Fbm(point * 0.7 + Randomize) - 1.0);
        latitude = saturate(latitude);
        if (latitude < latTropic - tropicWidth)
            climate = mix(climateTropic, climateEquator, (latTropic - tropicWidth - latitude) / latTropic) * 0.7;
        else if (latitude > latTropic + tropicWidth)
            climate =  mix(climateTropic, climatePole, (latitude - latTropic - tropicWidth) / (1.0 - latTropic))* 0.7;
        else
            climate =  climateTropic* 0.7;
    }
    else
    {
        latitude = 1.0 - point.x;
        latitude += 0.15 * (Fbm(point * 0.7 + Randomize) - 1.0);
        climate = mix(climateTropic, climatePole, latitude);
    }

    // Change climate with elevation
    float montHeight = max((height - seaLevel) / (snowLevel - seaLevel), 0.0);
    climate = mix(climate, climatePole, montHeight * montHeight);

    // Beach
    float beach = saturate((height / seaLevel - 1.0) * 50.0);
    climate = mix(0.375, climate, beach);

    // Dunes must be made off sand only
    slope *= step(dunesFraction, biome);

    // Ice caps
    float iceCap = saturate((latitude / latIceCaps - 0.97) * 100.0);
    climate = mix(climate, 1.0, iceCap);

    // Flatland climate distortion
    noiseOctaves = 4;
    p = point * colorDistFreq * 0.005 + ( 0.2 * (1.5 - RidgedMultifractal(point * 0.0005* hillsFreq/(hillsMagn*hillsMagn) + Randomize, 2.0))) + ( 0.05 * (1.5 - RidgedMultifractal(point * 0.005* hillsFreq/(hillsMagn*hillsMagn) + Randomize, 2.0)))+ ( 0.02 * (1.5 - RidgedMultifractal(point * 0.05* hillsFreq/(hillsMagn*hillsMagn) + Randomize, 2.0)));
    p += Fbm3D(p * 0.38) * 1.2; 
    float vary = (Fbm(p) + 0.7) * 0.7 * 0.5;
    climate += vary * beach * saturate(1.0 - 3.0 * slope) * saturate(1.0 - 1.333 * climate);

    // Color texture distortion
    noiseOctaves = 5;
    p = point * colorDistFreq * 2.3;
    p += Fbm3D(p * 0.5) * 1.2;
    vary = saturate((Fbm(p) + 0.7) * 0.7);

    surf = GetSurfaceColorAtlasSmart(climate, slope, vary);
    //surf = GetSurfaceColorAtlasSmooth(climate, slope, vary);
    //surf = GetSurfaceColorAtlas(climate, slope, vary);

    //surf.color = mix(surf.color, colorOverlay * biomeScale, 0.5);

    if (surfType <= 3)   // water mask for planets with oceans
        surf.color.a += saturate((seaLevel - height) * 200.0);

    return surf.color;
}

void main()
{
    vec3  point = GetSurfacePoint();
    float height, slope;
    GetSurfaceHeightAndSlope(height, slope);
    gl_FragColor = ColorMapTerra(point, height, slope);
}

#endif
